@@ -1598,7 +1598,7 @@ template <typename InputIt> InputIt eat_file(InputIt first, InputIt last) {
   for (; p != first && *(p - 1) != '/'; --p)
     ;
   if (p == first) {
-    // this should not happend in normal case, where we expect path
+    // this should not happened in normal case, where we expect path
     // starts with '/'
     *first++ = '/';
     return first;
@@ -1624,9 +1624,9 @@ StringRef path_join(BlockAllocator &balloc, const StringRef &base_path,
                     const StringRef &base_query, const StringRef &rel_path,
                     const StringRef &rel_query) {
   auto res = make_byte_ref(
-      balloc,
-      std::max(static_cast<size_t>(1), base_path.size()) + rel_path.size() + 1 +
-          std::max(base_query.size(), rel_query.size()) + 1);
+      balloc, std::max(static_cast<size_t>(1), base_path.size()) +
+                  rel_path.size() + 1 +
+                  std::max(base_query.size(), rel_query.size()) + 1);
   auto p = res.base;
 
   if (rel_path.empty()) {
